(*******************************************************************************
 *                           "whatsnew"
 *  An Utility program which find html files modified lateley.
 *              Copyright (C) 2001-2004  Fumio KAWAMATA
 *       The "whatsnew" is free software distributed under GNU GPL
 ******************************************************************************
 *)
unit whatsnew_main;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QButtons, QExtCtrls, QComCtrls, QMenus, QTypes,
  IniFiles, QMask, QActnList, QImgList,whatsnew_datamodule,Qt,QClipbrd;

type
  TFormMain = class(TForm)
    BitBtnAbort:                      TBitBtn;
    BitBtnExit:                       TBitBtn;
    BitBtnGetIt:                      TBitBtn;
    BitBtnSelectDir:                  TBitBtn;
    EditBaseDir:                      TEdit;
    EditTemplate:                     TEdit;
    ImageDays:                        TImage;
    ImageIgnorFiles:                  TImage;
    ImageIgnorSubdirectories:         TImage;
    ImagePeriod:                      TImage;
    ImageResult:                      TImage;
    ImageTargetDirectory:             TImage;
    ImageTargetFiles:                 TImage;
    ImageTemplate:                    TImage;
    MainMenu:                         TMainMenu;
    MainMenuDisplay:                  TMenuItem;
    MainMenuDisplayCfgFileName:       TMenuItem;
    MainMenuDisplayHomeDir:           TMenuItem;
    MainMenuDisplayIniFilename:       TMenuItem;
    MainMenuEdit:                     TMenuItem;
    MainMenuEditCopyToClipboard:      TMenuItem;
    MainMenuEditFont:                 TMenuItem;
    MainMenuFile:                     TMenuItem;
    MainMenuFileExit:                 TMenuItem;
    MainMenuFileGetIt:                TMenuItem;
    MainMenuFileSaveCfg:              TMenuItem;
    MainMenuFileSelectBaseDirectory:  TMenuItem;
    MainMenuHelp:                     TMenuItem;
    MainMenuHelpAbout:                TMenuItem;
    MainMenuOption:                   TMenuItem;
    MainMenuOptionWordWrap:           TMenuItem;
    MeinMenuOpenCfg:                  TMenuItem;
    MemoIgnorDirectories:             TMemo;
    MemoIgnorFiles:                   TMemo;
    MemoResult:                       TMemo;
    MemoTargetFiles:                  TMemo;
    PanelButtons:                     TPanel;
    PanelClient:                      TPanel;
    PanelIgnorFiles:                  TPanel;
    PanelIgnorSubdirectories:         TPanel;
    PanelPeriod:                      TPanel;
    PanelTargetDir:                   TPanel;
    PanelTargetFiles:                 TPanel;
    PanelTemplate:                    TPanel;
    PanelTop:                         TPanel;
    ProgressBar:                      TProgressBar;
    SpinEditPeriod:                   TSpinEdit;
    StatusBar:                        TStatusBar;
    procedure BitBtnAbortClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MainMenuEditCopyToClipboardClick(Sender: TObject);
    procedure MainMenuEditFontClick(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure MainMenuFileGetItClick(Sender: TObject);
    procedure MainMenuFileSaveCfgClick(Sender: TObject);
    procedure MainMenuFileSelectBaseDirectoryClick(Sender: TObject);
    procedure MainMenuHelpAboutClick(Sender: TObject);
    procedure MainMenuOpenCfgClick(Sender: TObject);
    procedure MainMenuOptionWordWrapClick(Sender: TObject);
    procedure MeinMenuOpenCfgClick(Sender: TObject);
  private
    function CopyToMemoResult: Integer;
    function GetFileList: Integer;
    function GetFileListSub(TargetDir:string): Integer;
    function GetWindowState: TWindowState;
    function SortByTime(var Names, Times: TStrings): Integer;
    procedure ReadFormStatusFromIniFile(Form: TForm);
    procedure ReadGeneralInformationFromIniFile;
    procedure ReadOptionsFromFile(OptFileFullPath: string);
    procedure SetStatusOfControls(Enabled: Boolean);
    procedure SetWordWrapRelatedOptions;
    procedure WriteFormStatusToIniFile(Form: TForm);
    procedure WriteGeneralInformationToIniFile;
    procedure WriteOptionsToFile(OptFileFullPath: string);
end;

var
  FormMain: TFormMain;

implementation

{$R *.xfm}

uses htmltitle, verinfo;

const
  {$IFDEF MSWINDOWS}
    Delimiter = '\';
    IniFileName = 'whatsnew.ini';
    InitialDir = 'C:\';
    CR = chr($0D)+chr($0A);
  {$ENDIF}
  {$IFDEF LINUX}
    Delimiter = '/';
    IniFileName = '.whatsnewrc';
    InitialDir = '/';
    CR = chr($0A);
  {$ENDIF}

type
  FileListRecType = record
    BaseDir:      string;
    Count:        Integer;
    IgnorDir:     TStrings;
    IgnorFile:    TStrings;
    Names:        TStrings;
    Recursive:    Boolean;
    Status:       TStrings;
    TargetFile:   TStrings;
    TimeCheck:    Boolean;
    Times:        TStrings;
    TimeStamp:    TDateTime;
  end;



var
    AbortFlag:        Boolean;
    FileListRec:      FileListRecType;
    HomeDir:          String;
    IniFileFullPath:  String;
    Msg:              TMessage = (Msg: CM_PARENTFONTCHANGED);
    Names:            TStrings;
    Times:            TStrings;



function TFormMain.GetWindowState: TWindowState;
begin
  if QWidget_isMaximized(Self.Handle) then
    Result := wsMaximized
  else if QWidget_isMinimized(Self.Handle) then
    Result := wsMinimized
  else
    Result := wsNormal;
end;



procedure TFormMain.ReadOptionsFromFile(OptFileFullPath: string);
var
  AppIni : TMemIniFile;
  I:       Integer;
  N:       Integer;
begin
  AppIni:=TMemIniFile.Create(OptFileFullPath);
  EditBaseDir.Text:=AppIni.ReadString('Options', 'BaseDir', InitialDir);
  EditTemplate.Text:=AppIni.ReadString('Options', 'Template', '<P>%D %A</P>');
  SpinEditPeriod.Value:=AppIni.ReadInteger('Options', 'Period', 30);

  N:=AppIni.ReadInteger('TargetFiles','NumberOfTargetFiles', 0);
  if N>0 then
  begin
    MemoTargetFiles.Lines.Clear;
    for I:=1 to N do
      MemoTargetFiles.Lines.Add(AppIni.ReadString('TargetFiles',IntToStr(I), ''));
  end;

  N:=AppIni.ReadInteger('IgnorDirectories','NumberOfIgnorDirectories', 0);
  MemoIgnorDirectories.Lines.Clear;
  if N>0 then
  begin
    for I:=1 to N do
      MemoIgnorDirectories.Lines.Add(AppIni.ReadString('IgnorDirectories',IntToStr(I), ''));
  end;

  N:=AppIni.ReadInteger('IgnorFiles','NumberOfIgnorFiles', 0);
  MemoIgnorFiles.Lines.Clear;
  if N>0 then
  begin
    for I:=1 to N do
      MemoIgnorFiles.Lines.Add(AppIni.ReadString('IgnorFiles',IntToStr(I), ''));
  end;
//  AppIni.UpdateFile;
  AppIni.Free;
end;


procedure TFormMain.ReadGeneralInformationFromIniFile;
var
  AppIni :   TMemIniFile;
  I:         Integer;
  FontStyle: Byte;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  Application.Font.Name:=AppIni.ReadString('Font', 'FontName', 'Times New Roman');
  FontStyle:=AppIni.ReadInteger('Font', 'FontStyle', 0);
  Application.Font.Style:=TFontStyles(FontStyle);
  Application.Font.Size:=AppIni.ReadInteger('Font', 'FontSize', 9);
  Application.Font.Charset:=TFontCharSet(AppIni.ReadInteger('Font', 'CharacterSet', 0));
    for I:= 0 to Screen.FormCount-1 do
      Screen.Forms[I].Dispatch(Msg);
  MainMenuOptionWordWrap.Checked:=AppIni.ReadBool('General', 'WordWrap', True);
  SetWordWrapRelatedOptions;
  AppIni.Free;
end;


procedure TFormMain.WriteGeneralInformationToIniFile;
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteString('Font', 'FontName', Application.Font.Name);
  AppIni.WriteInteger('Font', 'FontStyle', Byte(Application.Font.Style));
  AppIni.WriteInteger('Font', 'FontSize', Byte(Application.Font.Size));
  AppIni.WriteInteger('Font', 'CharacterSet', Byte(Application.Font.Charset));
  AppIni.WriteBool('General', 'WordWrap', MainMenuOptionWordWrap.Checked);
  AppIni.UpdateFile;
  AppIni.Free;
end;


procedure TFormMain.WriteOptionsToFile(OptFileFullPath: string);
var
  AppIni : TMemIniFile;
  I:       Integer;
  N:       Integer;
begin
  AppIni:=TMemIniFile.Create(OptFileFullPath);
  AppIni.WriteString('Options', 'BaseDir', EditBaseDir.Text);
  AppIni.WriteInteger('Options', 'Period', SpinEditPeriod.Value);
  AppIni.WriteString('Options', 'Template', EditTemplate.Text);
  N:=MemoTargetFiles.Lines.Count;
  AppIni.WriteInteger('TargetFiles','NumberOfTargetFiles', N);
  if N>0 then
  begin
    for I:=1 to N do
      AppIni.WriteString('TargetFiles',IntToStr(I), MemoTargetFiles.Lines[I-1]);
  end;
  N:=MemoIgnorDirectories.Lines.Count;
  AppIni.WriteInteger('IgnorDirectories','NumberOfIgnorDirectories', N);
  if N>0 then
  begin
    for I:=1 to N do
      AppIni.WriteString('IgnorDirectories',IntToStr(I), MemoIgnorDirectories.Lines[I-1]);
  end;
  N:=MemoIgnorFiles.Lines.Count;
  AppIni.WriteInteger('IgnorFiles','NumberOfIgnorFiles', N);
  if N>0 then
  begin
    for I:=1 to N do
      AppIni.WriteString('IgnorFiles',IntToStr(I), MemoIgnorFiles.Lines[I-1]);
  end;
  AppIni.UpdateFile;
  AppIni.Free;
end;


procedure TFormMain.ReadFormStatusFromIniFile(Form: TForm);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  Form.WindowState:=TWindowState(AppIni.ReadInteger(Form.Name,'WindowState', 0));
  Form.Top:=AppIni.ReadInteger(Form.Name,'Top', Form.Top);
  Form.Left:=AppIni.ReadInteger(Form.Name,'Left', Form.Left);
  Form.Width:=AppIni.ReadInteger(Form.Name,'Width', Form.Width);
  Form.Height:=AppIni.ReadInteger(Form.Name,'Height', Form.Height);
  //AppIni.UpdateFile;
  AppIni.Free;
end;


procedure TFormMain.WriteFormStatusToIniFile(Form: TForm);
var
  AppIni : TMemIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteInteger(Form.Name,'WindowState', Integer(GetWindowState));
  AppIni.WriteInteger(Form.Name,'Top', Form.Top);
  AppIni.WriteInteger(Form.Name,'Left', Form.Left);
  AppIni.WriteInteger(Form.Name,'Width', Form.Width);
  AppIni.WriteInteger(Form.Name,'Height', Form.Height);
  AppIni.UpdateFile;
  AppIni.Free;
end;



function TFormMain.SortByTime(var Names, Times: TStrings): Integer;
var
  Count: Integer;
  I:     Integer;
  J:     Integer;
  Tmp:   String;
begin
  Result:=0;
  Count:=Names.Count;
  if Count<2 then Exit;
  ProgressBar.Min:=0;
  ProgressBar.Max:=Count-2;
  StatusBar.SimpleText:='Sorting...';
  for I:=0 to Count-2 do
    begin
      for J:=I+1 to Count-1 do
        begin
          if Times[J]>Times[I] then
          begin
            Tmp:=Names[I];
            Names[I]:=Names[J];
            Names[J]:=Tmp;
            Tmp:=Times[I];
            Times[I]:=Times[J];
            Times[J]:=Tmp;
            Application.ProcessMessages;
            if AbortFlag=True then
            begin
              Result:=-1;
              Break;
            end;
          end;
        end;
      ProgressBar.Position:=I;
      if AbortFlag=True then
      begin
        Result:=-1;
        Break;
      end;
    end;
  ProgressBar.Position:=0;
  StatusBar.SimpleText:='';
end;



function TFormMain.CopyToMemoResult: Integer;
var
    DateStr:          string;
    FileName:         string;
    I:                Integer;
    RelativePath:     string;
    S:                string;
    TimeStr:          string;
    Title:            string;
begin
//MessageDlg('Program0', mtInformation, [mbOK], 0);
  Result:=0;
  MemoResult.Lines.Clear;
  if Names.Count=0 then
    Exit;
//MessageDlg(IntToStr(ProgressBar.Min)+'||'+IntToStr(ProgressBar.Max), mtInformation, [mbOK], 0);
//MessageDlg('Program1', mtInformation, [mbOK], 0);
  ProgressBar.Enabled:=False;
  if ProgressBar.Max<1 then
    ProgressBar.Max:=1;
  ProgressBar.Min:=1;
//MessageDlg('Program2', mtInformation, [mbOK], 0);
  ProgressBar.Max:=Names.Count;
  ProgressBar.Enabled:=True;
  StatusBar.SimpleText:='Getting the list...';
//MessageDlg('Loop start', mtInformation, [mbOK], 0);
  for I:=0 to Names.Count-1 do
  begin
    FileName:=FileListRec.BaseDir+Delimiter+Names[I];
//MessageDlg('FileName='+FileName, mtInformation, [mbOK], 0);
    Title:=GetHTMLTitle(FileName);
    DateStr:=copy(Times[I], 1, 10);
    TimeStr:=copy(Times[I], 12, 8);
    RelativePath:=StringReplace(Names[I], '\', '/', [rfReplaceAll]);

    S:=EditTemplate.Text;
    S:=StringReplace(S, '%D', DateStr, [rfReplaceAll]);
    S:=StringReplace(S, '%T', TimeStr, [rfReplaceAll]);
    S:=StringReplace(S, '%P', RelativePath, [rfReplaceAll]);
    S:=StringReplace(S, '%t', Title, [rfReplaceAll]);
    S:=StringReplace(S, '%A', '<A HREF="'+RelativePath+'">'+Title+'</A>', [rfReplaceAll]);
    S:=StringReplace(S, '%a', '<a href="'+RelativePath+'">'+Title+'</a>', [rfReplaceAll]);
    MemoResult.Lines.Add(S);


    ProgressBar.Position:=I;
    Application.ProcessMessages;
    if AbortFlag=True then
    begin
      Result:=-1;
      Break;
    end;
  end;
  MemoResult.Lines.Add('');
  ProgressBar.Position:=1;
  StatusBar.SimpleText:='';
end;



procedure TFormMain.FormCreate(Sender: TObject);
var
  AppIni : TMemIniFile;
procedure CreateIniFile;
begin
  AppIni:=TMemIniFile.Create(IniFileFullPath);
  AppIni.WriteString('General','Name','whatsnew');
  AppIni.UpdateFile;
  AppIni.Free;
  MessageDlg('The program created "'+IniFileFullPath+'" file.',mtInformation,[mbOK],0,mbOK);
end;
begin
  FormMain.Width:=640;
  FormMain.Height:=480;
  FormMain.Position:=poScreenCenter;
  FormMain.Caption:=FormMain.Caption+'  '+GetVersionInfo('FileVersion');
  Names:=TStringList.Create;
  Times:=TStringList.Create;
  HomeDir:=GetEnvironmentVariable('HOME');
  if HomeDir='' then
  begin
    HomeDir:=ExtractFileDir(ParamStr(0));
  end;
  MainMenuDisplayHomeDir.Caption:='Home directory: '+HomeDir;
  MainMenuDisplayIniFilename.Caption:='Ini filename: '+IniFileName;
  IniFileFullPath:=IncludeTrailingPathDelimiter(HomeDir)+IniFileName;
  if FileExists(IniFileFullPath)=False then
  begin
    CreateIniFile;
//    FormMain.Position:=poDeskTopCenter;
  end;
  ReadOptionsFromFile(IniFileFullPath);
  ReadGeneralInformationFromIniFile;
  ReadFormStatusFromIniFile(FormMain);
end;



procedure TFormMain.MainMenuOpenCfgClick(Sender: TObject);
var
  FileName : String;
begin
  MyDM.OpenDialog.InitialDir:=HomeDir;
  MyDM.OpenDialog.Options:=[ofNoChangedir];
  if MyDM.OpenDialog.Execute then
  begin
    FileName := MyDM.OpenDialog.FileName;
    MainMenuDisplayCfgFileName.Caption:=FileName;
  end;
end;



procedure TFormMain.FormDestroy(Sender: TObject);
begin
  Names.Free;
  Times.Free;
  WriteFormStatusToIniFile(FormMain);
  WriteGeneralInformationToIniFile;
  WriteOptionsToFile(IniFileFullPath);
end;



procedure TFormMain.MainMenuHelpAboutClick(Sender: TObject);
var
  Mess : String;
begin
  Mess:=VerInfo.GetVersionInfo('ProductName')+CR
       +VerInfo.GetVersionInfo('FileDescription')+CR
       +'Version '+VerInfo.GetVersionInfo('FileVersion')+CR
       +VerInfo.GetVersionInfo('LegalCopyright')+CR
       +'Freesoftware distributed under the GNU GPL version 2'+CR
       +'Homepage: http://openlab.jp/fumio/';
  MessageDlg(Mess,mtInformation,[mbOK],0);
end;



procedure TFormMain.MainMenuFileSaveCfgClick(Sender: TObject);
begin
  MyDM.SaveDialog.Title:='Save Config File';
  MyDM.SaveDialog.InitialDir:=InitialDir;
  MyDM.SaveDialog.Filter:='INI files (*.ini)|Text files (*.txt)|All files (*.*)';
  if MyDM.SaveDialog.Execute=True then
  begin
    WriteOptionsToFile(MyDM.SaveDialog.FileName);
  end;
end;



procedure TFormMain.SetStatusOfControls(Enabled: Boolean);
begin
  MainMenuFile.Enabled:=Enabled;
  MainMenuEdit.Enabled:=Enabled;
  MainMenuDisplay.Enabled:=Enabled;
  MainMenuOption.Enabled:=Enabled;
  MainMenuHelp.Enabled:=Enabled;
  EditBaseDir.Enabled:=Enabled;
  SpinEditPeriod.Enabled:=Enabled;
  BitBtnSelectDir.Enabled:=Enabled;
  MemoTargetFiles.Enabled:=Enabled;
  MemoIgnorDirectories.Enabled:=Enabled;
  MemoIgnorFiles.Enabled:=Enabled;
  MemoResult.Enabled:=Enabled;
  EditTemplate.Enabled:=Enabled;
  BitBtnGetIt.Enabled:=Enabled;
  BitBtnExit.Enabled:=Enabled;
  BitBtnAbort.Enabled:=Enabled;
end;

procedure TFormMain.MainMenuFileGetItClick(Sender: TObject);
begin
  AbortFlag:=False;
  SetStatusOfControls(False);
  BitBtnAbort.Enabled:=True;
  FileListRec.BaseDir:=EditBaseDir.Text;
  FileListRec.TargetFile:=MemoTargetFiles.Lines;
  FileListRec.IgnorDir:=MemoIgnorDirectories.Lines;
  FileListRec.IgnorFile:=MemoIgnorFiles.Lines;
  FileListRec.Names:=Names;
  FileListRec.Times:=Times;
  FileListRec.Recursive:=True;
  FileListRec.TimeCheck:=True;
  FileListRec.TimeStamp:=Now-SpinEditPeriod.Value;
  FileListRec.Status:=MemoResult.Lines;
  MemoResult.Lines.Clear;
  GetFileList;
  SortByTime(Names,Times);
  CopyToMemoResult;
  if AbortFlag=True then
  begin
    MemoResult.Lines.Clear;
    MemoResult.Lines.Add('Aborted.');
  end;
  SetStatusOfControls(True);
  BitBtnAbort.Enabled:=False;
end;



procedure TFormMain.MainMenuEditFontClick(Sender: TObject);
var
  I: Integer;
begin
  MyDM.FontDialog.Font:=Application.Font;
  if MyDM.FontDialog.Execute=True then
  begin
    Application.Font := MyDM.FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
      Screen.Forms[I].Dispatch(Msg);
  end;
end;



procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Application.Terminate;
end;



procedure TFormMain.MainMenuFileSelectBaseDirectoryClick(Sender: TObject);
var
  Dir : String;
begin
  if SelectDirectory('Enter Base Directory.', '/', Dir) = True then
  begin
      EditBaseDir.Text:=Dir;
  end;
end;



procedure TFormMain.FormResize(Sender: TObject);
var
  TotalWidth: Integer;
  PanelWidth: Integer;
begin
  TotalWidth:=PanelTargetDir.Width;
  EditBaseDir.Width:=TotalWidth-50;
  BitBtnSelectDir.Left:=TotalWidth-40;
  TotalWidth:=PanelTop.Width;
  PanelWidth:=TotalWidth div 5;
  PanelTargetFiles.Width:=PanelWidth;
  PanelIgnorFiles.Width:=PanelWidth;
  PanelPeriod.Width:=PanelWidth;
  EditTemplate.Width:=PanelTemplate.Width-8;
  ProgressBar.Left:=StatusBar.Width-ProgressBar.Width-20;
end;



procedure TFormMain.MainMenuEditCopyToClipboardClick(Sender: TObject);
var
  I: Integer;
begin
  Clipboard.AsText:='';
  MemoResult.WordWrap:=False;
  for I:=0 to MemoResult.Lines.Count-1 do
  begin
    Clipboard.AsText:=Clipboard.AsText+MemoResult.Lines[I]+CR;
  end;
  if MainMenuOptionWordWrap.Checked=True then
  begin
    MemoResult.WordWrap:=True;
  end;

{
  MemoResult.WordWrap:=False;
  MemoResult.SelectAll;
  MemoResult.CopyToClipboard;
  MemoResult.UnSelect;
  if MainMenuOptionWordWrap.Checked=True then
  begin
    MemoResult.WordWrap:=True;
  end;
}
end;



procedure TFormMain.MeinMenuOpenCfgClick(Sender: TObject);
begin
  MyDM.OpenDialog.Title:='Open Config File';
  MyDM.OpenDialog.InitialDir:=InitialDir;
  MyDM.OpenDialog.Filter:='INI files (*.ini)|Text files (*.txt)|All files (*.*)';
  if MyDM.OpenDialog.Execute=True then
  begin
    ReadOptionsFromFile(MyDM.OpenDialog.FileName);
    MainMenuDisplayCfgFileName.Caption:='Config file name: '+MyDM.OpenDialog.FileName;
  end;
end;



procedure TFormMain.SetWordWrapRelatedOptions;
begin
  if MainMenuOptionWordWrap.Checked=True then
  begin
    MemoResult.WordWrap:=True;
    MemoResult.ScrollBars:=ssAutoVertical;
  end
  else
  begin
    MemoResult.WordWrap:=False;
    MemoResult.ScrollBars:=ssAutoBoth;
  end;
end;



procedure TFormMain.MainMenuOptionWordWrapClick(Sender: TObject);
begin
  if MainMenuOptionWordWrap.Checked=True then
  begin
    MainMenuOptionWordWrap.Checked:=False;
  end
  else
  begin
    MainMenuOptionWordWrap.Checked:=True;
  end;
  SetWordWrapRelatedOptions;
end;



procedure TFormMain.BitBtnAbortClick(Sender: TObject);
begin
  AbortFlag:=True;
end;
{
function IsMatch(RelativePath,MaskPath: string): Boolean;
var
  MaskPathWork:         string;
  PosOfMaskPath:        Integer;
  RelativePathWork:     string;
begin
  Result:=False;
  if (RelativePath='')or(MaskPath='') then
    Exit;
  if copy(MaskPath,1,1)='*' then
  begin
    if Length(MaskPath)>1 then begin
      MaskPathWork:=MaskPath;
      Delete(MaskPathWork,1,1);
      PosOfMaskPath:=pos(MaskPathWork,RelativePath);
      if PosOfMaskPath>0 then
      begin
        RelativePathWork:=RelativePath;
        Delete(RelativePathWork,1,PosOfMaskPath-1);
        if MaskPathWork=RelativePathWork then
          Result:=True;
      end;
    end;
  end
  else
  begin
    if RelativePath=MaskPath then
      Result:=True;
  end;
end;
}
function IsMatch(RelativePath,MaskPath: string): Boolean;
var
  FileName:             string;
  MaskPathWork:         string;
  PosOfMaskPath:        Integer;
//  RelativePathWork:     string;
begin
  Result:=False;
  if (RelativePath='')or(MaskPath='') then
    Exit;
  FileName:=ExtractFileName(RelativePath);
  MaskPathWork:=MaskPath;
  if copy(MaskPath,1,1)='*' then
  begin
    if Length(MaskPath)>1 then
      Delete(MaskPathWork,1,1);
    PosOfMaskPath:=pos(MaskPathWork, FileName);
    if PosOfMaskPath>1 then
    begin
      Delete(FileName, 1, PosOfMaskPath-1);
      if MaskPathWork=FileName then
        Result:=True;
    end;
  end
  else
    if FileName=MaskPath then
      Result:=True;
end;


function IsIgnorDir(Dir: string):Boolean;
var
    LineIndex:    Integer;
    Matched:      Boolean;
begin
  Result:=False;
  if FileListRec.IgnorDir.Count=0 then
  begin
    Exit;
  end;
  Result:=False;
  LineIndex:=0;
  Matched:=False;
  repeat
    if FileListRec.IgnorDir[LineIndex]=Dir then
    begin
      Result:=True;
      Matched:=True;
//MessageDlg('Matched', mtInformation, [mbOK], 0);
    end;
    inc(LineIndex);
  until (LineIndex>=FileListRec.IgnorDir.Count)or(Matched=True);
end;



function IsIgnorFile(RelativePath: string):Boolean;
var
  LineIndex: Integer;
  Matched: Boolean;
begin
  Result:=False;
  if FileListRec.IgnorFile.Count=0 then
  begin
    Exit;
  end;
  Result:=False;
  LineIndex:=0;
  Matched:=False;
  repeat
    if IsMatch(RelativePath, FileListRec.IgnorFile[LineIndex]) then
    begin
      Result:=True;
      Matched:=True;
//MessageDlg('Matched', mtInformation, [mbOK], 0);
    end;
    inc(LineIndex);
  until (LineIndex>=FileListRec.IgnorFile.Count)or(Matched=True);
end;



function IsTargetFile(RelativePath: string):Boolean;
var
  LineIndex: Integer;
  Matched: Boolean;
begin
  Result:=False;
  if FileListRec.TargetFile.Count=0 then
    Exit;
  LineIndex:=0;
  Matched:=False;
  repeat
    if IsMatch(RelativePath, FileListRec.TargetFile[LineIndex])=True then
    begin
      Result:=True;
      Matched:=True;
    end;
    inc(LineIndex);
  until (LineIndex>=FileListRec.TargetFile.Count)or(Matched=True);
end;



function IsInPeriod(FileDate: Integer):Boolean;
var
  DT : TDateTime;
begin
  DT:=FileDateToDateTime(FileDate);
  if DT>=FileListRec.TimeStamp then
    Result:=True
  else
    Result:=False;
end;



function TFormMain.GetFileListSub(TargetDir:string): Integer;
const
  {$IFDEF MSWINDOWS}
    Filter ='*.*';
  {$ENDIF}
  {$IFDEF LINUX}
    Filter ='*';
  {$ENDIF}
var
  SearchRec : TSearchRec;
  RelativePath : string;
  LengthOfBaseDir : Integer;
begin
  Result:=0;
  if FindFirst(TargetDir+Delimiter+Filter, faAnyFile, SearchRec)=0 then
  begin
    LengthOfBaseDir:=Length(FileListRec.BaseDir);
    try
      repeat
        if (SearchRec.Name='.')or(SearchRec.Name='..') then
          Continue;
          RelativePath:=TargetDir+Delimiter+SearchRec.Name;;
          Delete(RelativePath,1,LengthOfBaseDir);
          if RelativePath<>'' then
          begin
            if Copy(RelativePath,1,1)=Delimiter then Delete(RelativePath,1,1);
          end;
        if SearchRec.Attr and faDirectory=faDirectory then
        begin
          FileListRec.Status.Add(RelativePath);
          if FileListRec.Recursive=True then
          begin
            if IsIgnorDir(RelativePath)=False then
            begin
              GetFileListSub(TargetDir+Delimiter+SearchRec.Name);
            end
            else
            begin
              // Do nothing.
            end;
          end;
        end
        else
          begin
            if IsIgnorFile(RelativePath)=False then
            begin
              if IsTargetFile(RelativePath)=True then
              begin
                if IsInPeriod(SearchRec.Time)=True then
                begin
                  FileListRec.Names.Add(RelativePath);
                  FileListRec.Times.Add(FormatDateTime('yyyy/mm/dd hh:nn:ss',FileDateToDateTime(SearchRec.Time)));
                end;
              end;
            end;
          end;
        Application.ProcessMessages;
        if AbortFlag=True then
        begin
          Result:=-1;
          Exit;
        end;
      until FindNext(SearchRec)<>0;
    finally
      FindClose(SearchRec);
    end; {try}
  end; {if FindFirst(...)=0}
end;

function TFormMain.GetFileList: Integer;
begin
  StatusBar.SimpleText:='Getting file structure...';
  FileListRec.Names.Clear;
  FileListRec.Times.Clear;
  FileListRec.Count:=0;
  Result:=GetFileListSub(FileListRec.BaseDir);
  StatusBar.SimpleText:='';
end;




end.
